"""
Layered Histogram
=================
This example shows how to make a layered histogram in Altair.
"""

import pandas as pd
import altair as alt
import numpy as np
np.random.seed(42)

# Generating Data
df = pd.DataFrame({'Trial A':np.random.normal(0, 0.8, 1000),
                   'Trial B':np.random.normal(-2, 1, 1000),
                   'Trial C':np.random.normal(3, 2, 1000)})

# Tidying Data
df = pd.melt(df, id_vars=df.index.name,
             value_vars=df.columns,
             var_name = 'Experiment',
             value_name='Measurement')

chart = alt.Chart(df).mark_area(
    opacity=0.3,
    interpolate='step'
).encode(
    alt.X('Measurement', bin=alt.Bin(maxbins=100)),
    alt.Y('count(*):Q', stack=None),
    alt.Color(
        'Experiment',
        scale=alt.Scale(range=['#0000ff', '#008000', '#ff0000'])
    )
)



"""
LayerChart with Bar and Tick
----------------------------
This example shows how to layer two charts on top of one another.
"""

import altair as alt
import pandas as pd

data = pd.DataFrame({
    'project': ['a', 'b', 'c', 'd', 'e', 'f', 'g'],
    'score': [25, 57, 23, 19, 8, 47, 8],
    'goal': [25, 47, 30, 27, 38, 19,4]}
)

a = alt.Chart().mark_bar().encode(
    x='project',
    y='score'
)

b = alt.Chart().mark_tick(
    color='red',
).encode(
    x='project',
    y='goal'
)

chart = alt.layer(a, b).properties(
    data=data
).configure_tick(
    thickness=2,
    bandSize=35  # controls the width of the tick
).configure_scale(
    rangeStep=40  # controls the width of the bar
)



"""
Slope Graph
-----------------------
This example shows how to make Slope Graph.
"""

import altair as alt
from vega_datasets import data

source = data.barley()

# The year here is stored by pandas as an integer. When treating columns as dates,
# it is best to use either a string representation or a datetime representation.
source.year = source.year.astype(str)

chart = alt.Chart(source).mark_line().encode(
    x='year',
    y='median(yield)',
    color='site'
)



"""
Simple Scatter Plot
-------------------

A simple example of an interactive scatter plot using the well-known iris
dataset.
"""
# category: basic charts

from altair import Chart
from vega_datasets import data

iris = data.iris()

chart = Chart(iris).mark_point().encode(
    x='petalWidth',
    y='petalLength',
    color='species'
).interactive()



"""
Interactive Chart with Cross-Highlight
======================================
This example shows an interactive chart where selections in one portion of
the chart affect what is shown in other panels. Click on the bar chart to
see a detail of the distribution in the upper panel.
"""
# category: interactive

import altair as alt
from vega_datasets import data

pts = alt.selection(type="single", encodings=['x'])

rect = alt.Chart(data.movies.url).mark_rect().encode(
    alt.X('IMDB_Rating:Q', bin=True),
    alt.Y('Rotten_Tomatoes_Rating:Q', bin=True),
    alt.Color('count(*):Q',
        scale=alt.Scale(scheme='greenblue'),
        legend=alt.Legend(title='Total Records')
    )
)

circ = rect.mark_point().encode(
    alt.ColorValue('grey'),
    alt.Size('count(*):Q',
        legend=alt.Legend(title='Records in Selection')
    )
).transform_filter(
    pts.ref()
)

bar = alt.Chart(data.movies.url).mark_bar().encode(
    x='Major_Genre:N',
    y='count(*):Q',
    color=alt.condition(pts, alt.ColorValue("steelblue"), alt.ColorValue("grey"))
).properties(
    selection=pts,
    width=550,
    height=200
)

chart = alt.vconcat(
    rect + circ,
    bar
).resolve_legend(
    color="independent",
    size="independent"
)



"""
Utilities that use selenium + chrome headless to save figures
"""

import base64
import io
import json
import os
import tempfile

import six

from .importing import attempt_import
from .core import write_file_or_filename


HTML_TEMPLATE = """
<!DOCTYPE html>
<html>
<head>
  <title>Embedding Vega-Lite</title>
  <script src="https://cdn.jsdelivr.net/npm/vega@3.2"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-lite@2.3"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-embed@3.0.0"></script>
</head>
<body>
  <div id="vis"></div>
</body>
</html>
"""

EXTRACT_CODE = {
'png': """
        var spec = arguments[0];
        var mode = arguments[1]
        var done = arguments[2];

        if(mode === 'vega-lite'){
          // compile vega-lite to vega
          const compiled = vl.compile(spec);
          spec = compiled.spec;
        }

        new vega.View(vega.parse(spec), {
              loader: vega.loader(),
              logLevel: vega.Warn,
              renderer: 'none',
            })
            .initialize()
            .toCanvas()
            .then(function(canvas){return canvas.toDataURL('image/png');})
            .then(done)
            .catch(function(err) { console.error(err); });
        """,
'svg': """
        var spec = arguments[0];
        var mode = arguments[1];
        var done = arguments[2];

        if(mode === 'vega-lite'){
          // compile vega-lite to vega
          const compiled = vl.compile(spec);
          spec = compiled.spec;
        }

        new vega.View(vega.parse(spec), {
              loader: vega.loader(),
              logLevel: vega.Warn,
              renderer: 'none',
            })
            .initialize()
            .toSVG()
            .then(done)
            .catch(function(err) { console.error(err); });
        """}

def save_spec(spec, fp, mode=None, format=None, driver_timeout=10):
    """Save a spec to file

    Parameters
    ----------
    spec : dict
        a dictionary representing a vega-lite plot spec
    fp : string or file-like object
        the filename or file object at which the result will be saved
    mode : string or None
        The rendering mode ('vega' or 'vega-lite'). If None, the mode will be
        inferred from the $schema attribute of the spec, or will default to
        'vega' if $schema is not in the spec.
    format : string (optional)
        the file format to be saved. If not specified, it will be inferred
        from the extension of filename.
    driver_timeout : int (optional)
        the number of seconds to wait for page load before raising an
        error (default: 10)

    Note
    ----
    This requires the pillow, selenium, and chrome headless packages to be
    installed.
    """
    # TODO: allow package versions to be specified
    # TODO: detect & use local Jupyter caches of JS packages?

    if format is None and isinstance(fp, six.string_types):
        format = fp.split('.')[-1]

    if format not in ['png', 'svg']:
        raise NotImplementedError("save_spec only supports 'svg' and 'png'")

    webdriver = attempt_import('selenium.webdriver',
                               'save_spec requires the selenium package')
    Options = attempt_import('selenium.webdriver.chrome.options',
                             'save_spec requires the selenium package').Options

    if mode is None:
        if '$schema' in spec:
            mode = spec['$schema'].split('/')[-2]
        else:
            mode = 'vega'
    if mode not in ['vega', 'vega-lite']:
        raise ValueError("mode must be 'vega' or 'vega-lite'")

    try:
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        driver = webdriver.Chrome(chrome_options=chrome_options)
        driver.set_page_load_timeout(driver_timeout)

        try:
            fd, htmlfile = tempfile.mkstemp(suffix='.html', text=True)
            with open(htmlfile, 'w') as f:
                f.write(HTML_TEMPLATE)
            driver.get("file://" + htmlfile)
            render = driver.execute_async_script(EXTRACT_CODE[format],
                                                 spec, mode)
        finally:
            os.remove(htmlfile)
    finally:
        driver.close()

    if format == 'png':
        img_bytes = base64.decodebytes(render.split(',')[1].encode())
        write_file_or_filename(fp, img_bytes, mode='wb')
    else:
        img_bytes = render.encode()
        write_file_or_filename(fp, img_bytes, mode='wb')



import os
from os.path import join, dirname, abspath

import pytest

EXAMPLE_DIR = abspath(join(dirname(__file__), '..'))


def iter_example_filenames():
    for filename in os.listdir(EXAMPLE_DIR):
        if filename.startswith('__'):
            continue
        if not filename.endswith('.py'):
            continue
        yield filename


@pytest.mark.parametrize('filename', iter_example_filenames())
def test_examples(filename):
    with open(join(EXAMPLE_DIR, filename)) as f:
        source = f.read()
    globals_ = {}
    exec(source, globals_)

    if 'chart' not in globals_:
        raise ValueError("Example file should define a chart variable")
    chart = globals_['chart']
    dct = chart.to_dict()



"""
Cumulative Wikipedia Donations
==============================

This chart shows cumulative donations to Wikipedia over the past 10 years. This chart was inspired by https://www.reddit.com/r/dataisbeautiful/comments/7guwd0/cumulative_wikimedia_donations_over_the_past_10/ but using lines instead of areas.
Data comes from https://frdata.wikimedia.org/.
"""

import altair as alt

data = "https://frdata.wikimedia.org/donationdata-vs-day.csv"

chart = alt.Chart(data).mark_line().encode(
    alt.X(
        'date:T', timeUnit='monthdate',
        axis=alt.Axis(format='%B', title='Month')
    ),
    alt.Y(
        'max(ytdsum):Q', stack=None,
        axis=alt.Axis(title='Cumulative Donations')
    ),
    alt.Color('date:O', timeUnit='year', legend=alt.Legend(title='Year')),
    alt.Order('data:O', timeUnit='year')
)
